#include <ch32v00x.h>
#include <debug.h>

static const char keys[][3] = {
	{ '1', '2', '3', },
	{ '4', '5', '6', },
	{ '7', '8', '9', },
	{ '*', '0', '#', },
};

int main() {
	USART_Printf_Init(115200);

	// On initialise le GPIO
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
	GPIO_InitTypeDef gpioInit = { 0 };
	// PC0..3 correspondent aux ranges (entres), de haut en bas.
	gpioInit.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3;
	gpioInit.GPIO_Mode = GPIO_Mode_IPD;
	GPIO_Init(GPIOC, &gpioInit);
	// PC5..7 correspondent aux colonnes (sorties), de gauche  droite.
	gpioInit.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;
	gpioInit.GPIO_Mode = GPIO_Mode_Out_PP;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOC, &gpioInit);

	int currentColumn = 0;

	while (1) {
		// Le principe est d'envoyer un niveau logique haut alternativement
		// sur chaque sortie. Si on retrouve ce niveau sur une entre, c'est
		// que la touche correspondante est appuye.
		switch (currentColumn) {
		case 0:
			GPIO_Write(GPIOC, 0x20);
			break;

		case 1:
			GPIO_Write(GPIOC, 0x40);
			break;

		case 2:
			GPIO_Write(GPIOC, 0x80);
			break;
		}

		// La lecture du port renvoie l'tat de toutes les lignes,
		// donc sorties comprises.
		int key = GPIO_ReadInputData(GPIOC);

		// On regarde si on trouve un niveau logique haut sur une des entres.
		if (key & 0x0f) {
			// On recherche la position du "1" dans les entres pour trouver
			// la range.
			int row = 0;
			for (int v = key; !(v & 1); v >>= 1, row++);
			// On recherche la position du "1" dans les sorties pour trouver
			// la colonne, en ignorant PC4 que nous n'utilisons pas.
			int column = 0;
			for (int v = key; !(v & 0x20); v >>= 1, column++);
			// On affiche range, colonne et caractre correspondant.
			printf("rangee = %d, colonne = %d, touche = %c\r\n", row, column, keys[row][column]);
		}

		// On passe  la colonne suivante.
		currentColumn++;

		if (currentColumn > 2) {
			currentColumn = 0;
		}
	}
}
